/*
 * Command grammar:
 *
 * simple_command -> word... CTRL(opt) ;
 *
 * PIPE word
 *
 * CTRL -> num(opt) ">" WS(opt) pathname
 *       | num(opt) "<" WS(opt) pathname
 *	 | num(opt) ">>" WS(opt) pathname
 *	 | num(opt) "<|" word
 *	 | num(opt) ">|" word
 *	 | num(opt) ">&" num
 *	 | num(opt) "<&" num
 *	 ;
 *
 * we do not support HERE DOCUMENTS (yet)
 */

enum redir_type { rnone, rfile, rdesc, rpipe, rclose };
enum redir_dir { rin, rout, rfappend };

struct redir_op {
	struct redir_op	*next;
	enum redir_dir	dir;
	enum redir_type	src_type;
	int		target_desc;
	union {
		int	desc;
		char	*name;			/* pathname or pipe name */
	} src;
};

struct cmd {
	struct redir_op	*rop;	/* LIFO */
	int		background;
	int		nwords, szwords;
	char		**words;
};

enum lexer_state {
	LEXER_NORM,
	LEXER_NORM_BSLASH,
	LEXER_NORM_DLR,
	LEXER_NORM_DLR2,
	LEXER_DBL,
	LEXER_DBL_BSLASH,
	LEXER_DBL_DLR,
	LEXER_DBL_DLR2,
	LEXER_SGL,
	LEXER_NORM_LESS,
	LEXER_NORM_GTR,
	LEXER_NORM_LESS_AMP,
	LEXER_NORM_GTR_AMP
};
/*
 * strictly speaking, we do a little more than lexical analysis here.
 */
struct lexer {
	FILE			*stream;
	enum lexer_state	state;
	enum redir_type		word_dest;
	enum redir_dir		direction;
	int			target;
	struct strbuf		vname;
};
void		lexer_constr(struct lexer	*lp,
			     FILE		*str);
struct lexer	*lexer_constr_ptr(FILE	*str);
void		lexer_destr(struct lexer	*lp);
void		lexer_destr_ptr(struct lexer	*lp);
void 		cmd_constr(struct cmd	*cmdp);
struct cmd	*cmd_constr_ptr(void);
struct redir_op	*redir_op_constr_ptr(void);
void		redir_op_destr_ptr(struct redir_op	*rop);
void		cmd_destr(struct cmd	*cmdp);
void		cmd_destr_ptr(struct cmd	*cmdp);

struct cmd	*get_simple_cmd(struct lexer	*in_lexer);
